/* filename:    dsgepm.g
** description: The program maximizes prior*likelihood for the 
**              DSGE model
** created:     02/06/02
*/

new;
library pgraph, user;
cls;

format /mat /on /mb1 /ros 16,8;


/* Global defaults
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\mispol\gauss-ds\pathspec.g
#include c:\projects\active\mispol\gauss-ds\bivspec.g

/******************************************************** 
** Import data
** series (nobs,5)
*/
#include c:\projects\active\mispol\gauss-ds\loaddata.g

nobs  = rows(series);       /* number of observations          */
YY    = series;   

(0.25~1~0.25)*(stdc(YY)^2);
end;

/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ "pri.out";
load path = ^priopath prior[npara,3] = ^priorfile;

pmean  = prior[1:npara,1];
pstdd  = prior[1:npara,2];
pshape = prior[1:npara,3];
pshape = pshape.*para_maskinv[.,mspec];

/******************************************************** 
** Starting Values for Maximization
*/

psi1  = 1.8;    /* policy rule: inflation */
psi2  = 0.25;   /* policy rule: output gap */
rhoR  = 0.6;    /* policy rule: smoothing */
   
lngamm = 2;     /* technology growth */
lnpist = 4;     /* ss inflation */
lnrrst = 2;     /* ss real rate */
kap   = 0.3;    /* PC slope */
tau   = 0.8;    /* substitution elasticity */
   
rhog = 0.8;    /* Persistence: government spending */
rhoz  = 0.3;    /* Persistence: technology growth*/

sig_r = 1;
sig_g = 1;
sig_z = 1;

para  = psi1 | psi2 | rhoR |
        lngamm | lnpist | lnrrst | kap | tau |
        rhog | rhoz |
        sig_r | sig_g | sig_z;

para  = para.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];

/* Load transformation scheme for parameters
*/
transfile = "trans.out";
load path=^priopath _trspec[npara,4] = ^transfile;
_trspec[.,1] = _trspec[.,1].*para_maskinv[.,mspec];

/******************************************************** 
**      Calculate posterior density at starting values
**
*/
cls;
"Prior Density at Starting Value:" priodsge(para,pmean,pstdd,pshape);
lnpy = -objfcn(invtrans(para));
"Posterior at Starting Value:    " lnpy;
"Press key to continue";
w = keyw;
cls;

/********************************************************
**       Maximize the posterior density
**
*/
cc = 0;
x0 = invtrans(para) + cc* ( para_maskinv[.,mspec].*rndn(npara,1) );

H0= eye(npara)*1E-4;
nit = 1000;
crit= 1E-7;

/* Use personal optimization procedure
*/
{fh,xh,g,H,itct,fcount,retcode} = csminwel(x0,H0,crit,nit,&objfcn);

/********************************************************
** Save parameter estimates
*/
paraest = trans(xh);
paraest = real( paraest.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec] );

oparaest         = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ "pm";
create fhparaest = ^oparaest with PARAEST, 1, 8;
wr               = writer(fhparaest,paraest);
closeall fhparaest;   


/********************************************************
**       Estimation Output
**
*/
{lnpy, retcode, obsmean, obsvar} = evaldsge( paraest,mspec,YY);
lnprio  = priodsge( paraest, pmean, pstdd, pshape);

/* Compute gradient in terms of model parameters
*/
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;
"Model              " mspec;
"Dataset            " datasel;
"===================================";
"Posterior Mode     " (lnpy+lnprio);
"Likelihood at Mode " lnpy;
"Prior at Mode      " lnprio;
" ";
"Parameter   | Estimate |  Start-V  | Prior Mean | Prior Stdd | Gradient";
outmat = para_names'~paraest~trans(x0)~pmean~pstdd~gradmod;
let mask[1,6] = 0 1 1 1 1 1;
let fmt[6,3] =
   "-*.*s " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4
   "*.*lf " 8 4;

d = printfm(outmat,(0~1~1~1~1~1),fmt);

if lnpy == -1E6 ;
   end;
endif;

/********************************************************
**       Compute Residuals
**
*/

/* Moments */

resid1 = YY - obsmean;
resid1_m1 = meanc(resid1)';
resid1_m2 = meanc( (resid1 - resid1_m1)^2 )';
resid1_m3 = meanc( (resid1 - resid1_m1)^3 )';
resid1_m4 = meanc( (resid1 - resid1_m1)^4 )';

resid2 = (YY - obsmean)./sqrt(obsvar);
resid2_m1 = meanc(resid1)';
resid2_m2 = meanc( (resid2 - resid2_m1)^2 )';
resid2_m3 = meanc( (resid2 - resid2_m1)^3 )';
resid2_m4 = meanc( (resid2 - resid2_m1)^4 )';

/* Autocorrelation */

resid1_ar = zeros(1,nvar);
resid2_ar = zeros(1,nvar);

ind = 1;
do until ind > nvar;
   resid1_ar[1,ind]=inv(resid1[1:nobs-1,ind]'*resid1[1:nobs-1,ind])
                    *resid1[1:nobs-1,ind]'*resid1[2:nobs,ind];
   resid2_ar[1,ind]=inv(resid2[1:nobs-1,ind]'*resid2[1:nobs-1,ind])
                    *resid2[1:nobs-1,ind]'*resid2[2:nobs,ind];
   ind = ind+1;
endo;


"Raw Residuals";
"-------------";
"Mean:" resid1_m1;
"Stdd:" sqrt(resid1_m2);
"Skew:" resid1_m3./resid1_m2^(3/2);
"Kurt:" resid1_m4./(3*resid1_m2^2);
"Corr:" resid1_ar;
"";
"Standardized Residuals";
"----------------------";
"Mean:" resid2_m1;
"Stdd:" sqrt(resid2_m2);
"Skew:" resid2_m3./resid2_m2^(3/2);
"Kurt:" resid2_m4./(3*resid2_m2^2);
"Corr:" resid2_ar;
"";

titlestr = "Output"  $|
           "Inflation" $|
           "Interest Rate" $|
           "Exchange Rate" $|
           "Terms of Trade" ;

ystr     = "Raw Residual " $|
           "Stdd Residual"; 

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
          
begwind;
window(2,nvar,0);


ser_ind = 1;
do until ser_ind > nvar;

   ylabel(ystr[1]);
   title(titlestr[ser_ind]);
   xy(ti,resid1[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

ser_ind = 1;
do until ser_ind > nvar;

   ylabel(ystr[2]);
   title(titlestr[ser_ind]);
   xy(ti,resid2[.,ser_ind]);
 
   ser_ind = ser_ind+1;
   nextwind;

endo;

endwind;
end;


/****************************************************/
/*                 PROCEDURES                       */
/****************************************************/

proc (1) = objfcn(para);
local lnpY, lnprio, obsmean, obsvar, modelpara, retcode;

/* likelihood 
*/
modelpara = trans(para);
modelpara = modelpara.*para_maskinv[.,mspec] + para_fix[.,mspec].*para_mask[.,mspec];
 
{lnpY,retcode,obsmean,obsvar} = evaldsge(modelpara,mspec,YY); 

/* Evaluate the Prior distribution
*/
lnprio = priodsge(modelpara, pmean, pstdd, pshape);

retp(real(-lnpY-lnprio));  /* We minize the inverse of the likelihood fcn */
endp;

#include c:\projects\active\mispol\gauss-ds\dsgemod.src;
#include c:\projects\active\mispol\gauss-ds\dsgelh.src;
